package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Simulating car battery power.</p>
 *
 */
public class BatteryPlanner {

    /**
     * <p>How many kilometer (Km) does 1 Kwh (Kilowatt hour) of energy allow you to drive?
     *    See {@link #BatteryPlanner(int, int)} for details.</p>
     */
    public final int kilometerByKwh;

    /**
     * The battery's capacity in kilowatt-hours (kwh). See {@link #BatteryPlanner(int, int)} for details.
     */
    public final int capacity;

    private double stateOfCharge;

    /**
     * <p>Offering a certain amount of energy for charging.</p>
     *
     * <p>The battery may be charged up to its capacity thereby raising its internal state of charge. </p>
     *
     * <ul>
     *     <li>In case the offered amount is too large the excess value is being returned as a positive value.</li>
     *
     *     <li>A negative return value indicates the amount still being required for the battery's complete recharge.</li>
     *
     * </ul>
     *
     * <p>Example: We assume a battery having a capacity of 60 kwh being charged with 20 kwh. We may
     * thus charge 60 kwh - 20 kwh = 40 kwh at maximum.
     *
     * <ul>
     *     <li>When offering 50 kwh the excess of 50kwh - 40kwh = 10 kwh cannot be used for charging and will thus be
     *     returned.</li>
     *
     *     <li>Offering just 30 kwh results in an incomplete 50Kwh state of charge and an excess of -10 Kwh. This
     *     indicates another 10Kwh being required for a complete battery charge.</li>
     * </ul>
     *
     * @param chargeOffer The amount of energy in kwh being offered for charging.
     *
     * @return Positive values indicate excess energy and negative values indicate lack of energy with respect to
     *         the battery's capacity and given state of charge.
     */
    public double charge (final double chargeOffer) {
        final double excess = chargeOffer  + stateOfCharge - capacity;
        if (excess <= 0) {
            stateOfCharge += chargeOffer;
        } else {
            stateOfCharge = capacity;
        }
        return excess;
    }

    /**
     * <p>Driving an intended distance:</p>
     *
     * <p>Depending on the battery's state of charge either the intended distance or a smaller value
     *     being covered by the stored amount of Kwh will be returned. We consider an intended
     *     distance of 100 Km:</p>
     *
     * <ul>
     *     <li>The battery allows for 200 Km. In this case the full 100 Km intended distance is being
     *     returned and the battery's state of charge will be reduced accordingly.</li>
     *
     *     <li>The battery's state of charge only allows for 40 Km. This value is being returned an the
     *     battery's state of charge will drain to zero.</li>
     * </ul>
     *
     * @param intendedDistance The intended driving distance in kilometer (km).
     * @return The distance actually being covered.
     */
    public double drive (final double intendedDistance) {
        final double currentRange = stateOfCharge * kilometerByKwh;

        if (intendedDistance <= currentRange) {
            stateOfCharge -= intendedDistance / kilometerByKwh;
            return intendedDistance;
        } else {
            stateOfCharge = 0;
            return currentRange;
        }
    }

    /**
     * <p>Return the battery's state of charge in terms of {@link BatteryState} categories.</p>
     *
     * @return An enum value from {@link BatteryState}.
     */
    public BatteryState getState() {
        return BatteryState.getState(stateOfCharge * 100. / capacity);
    }

    /**
     * <p>A car / battery system of given capacity being fully charged initially.</p>
     *
     * <p>An electric car's driving range is (among other factors) being determined by:</p>
     *
     * <ul>
     *     <li>Battery capacity being measured in kilowatt hours (kwh).</li>
     *     <li>Its weight and other factors demanding a certain amount of energy per driven kilometer.</li>
     * </ul>
     *
     * <p>Example: A car delivering 200 km before completely draining a battery of 50 kwh requires
     *    1 Kwh per 4Km of driving.</p>
     *
     * @param capacity See {@link #capacity}.
     * @param kilometerByKwh See {@link #kilometerByKwh}.
     */
    public BatteryPlanner(final int capacity, final int kilometerByKwh) {
        this.capacity = capacity;
        this.kilometerByKwh = kilometerByKwh;
        this.stateOfCharge = capacity;
    }
}
